------------------------------
-- Localized Lua globals
------------------------------
local _G = getfenv(0)

--local tostring = _G.tostring
--local tonumber = _G.tonumber

local pairs, ipairs = _G.pairs, _G.ipairs
local select = _G.select

--local table   = _G.table
--local tconcat = table.concat
--local tinsert = table.insert

--local bit = _G.bit

local string    = _G.string
local strformat = string.format
--local strfind   = string.find
--local strmatch  = string.match
--local strlower  = string.lower

------------------------------
-- Localized Blizzard API
------------------------------

------------------------------
-- AddOn Namespace
------------------------------
local LibStub = _G.LibStub
local MODNAME = "Skill Guide"
local addon   = LibStub("AceAddon-3.0"):NewAddon(MODNAME, "AceConsole-3.0", "AceEvent-3.0")
_G.SkillGuide = addon

local L = LibStub("AceLocale-3.0"):GetLocale(MODNAME)

-- Get the private table
local private = select(2, ...)

-- Global Options
addon.optionsFrame = {}

------------------------------
-- Check for the required libriaries, Blizzard
-- already checked for dependencies
------------------------------
local MissingLibs
do
  local REQUIRED_LIBS = {
    "AceLocale-3.0",
  }
  function MissingLibs()
    local missing = false
    
    for idx, lib in ipairs(REQUIRED_LIBS) do
      if not LibStub:GetLibrary(lib, true) then
        missing = true
        addon:Print(strformat(L["MISSING_LIB"], lib))
      end
    end
    
    return missing
  end
end -- do

if MissingLibs() then
  --@alpha@
  addon:Print("You are using an alpha version of Skill Guide. Not all required libs are installed. Please install the missing libraries and restart.")
  --@end-alpha@
  _G.SkillGuide = nil
  return
end

function addon:OnInitialize()
  addon:Print("Initializing...")
  
  local defaults = {
    global = {
      tradeskill = {},
    },
    profile = {
      frameopts = {},
    },
  }  
    
  self.db = LibStub("AceDB-3.0"):New("SkillGuideDB", defaults)
  if not self.db then
    self:Print("SavedVariables not loaded")
    return
  end
  
  local version = GetAddOnMetadata("SkillGuide", "Version")
  self.version = version
  
  self:SetupOptions()
  
  -- Register slash command
  self:RegisterChatCommand("skillguide", "ChatCommand")
  self:RegisterChatCommand("sg", "ChatCommand")
end

function addon:OnEnable()
  self:Print(MODNAME .. " enabled...")
  
  -- Register for events
  self:RegisterEvent("TRADE_SKILL_SHOW")
  self:RegisterEvent("TRADE_SKILL_CLOSE")
  self:RegisterEvent("TRADE_SKILL_UPDATE")
end

function addon:ChatCommand(input)
  -- Open the Blizzard Interface Addon Panel if there is no input
  -- or /sg faq
  if not input or (input and input:trim() == "") then
    InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
  elseif (input == strlower(L["About"])) then
    if (self.optionsFrame["About"]) then
      InterfaceOptionsFrame_OpenToCategory(self.optionsFrame["About"]
    else
      InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
    end
  elseif (input == strlower(L["Profile"])) then
    if (self.optionsFrame["Profile"]) then
      InterfaceOptionsFrame_OpenToCategory(self.optionsFrame["Profiles"]
    else
      InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
    end
  end
end

------------------------------
-- Event Handling Callbacks
------------------------------
do
  local GetTradeSkillListLink = _G.GetTradeSkillListLink
  local UnitName = _G.UnitName
  local GetRealmName = _G.GetRealmName
  local IsTradeSkillLinked = _G.IsTradeSkillLinked
  
  function addon:TRADE_SKILL_SHOW()
    -- If this is our skill, save it
    if not IsTradeSkillLinked() then
      local tradelink = GetTradeSkillListLink()
      if tradelink then
        local pname = UnitName("player")
        local prealm = GetRealmName()
        local tradename = GetTradeSkillLine()
        
        addon.db.global.tradeskill = addon.db.global.tradeskill or {}
        addon.db.global.tradeskill[prealm] = addon.db.global.tradeskill[prealm] or {}
        addon.db.global.tradeskill[prealm][pname] = adddon.db.global.tradeskill[prealm][pname] or {}
        addon.db.global.tradeskill[prealm][pname][tradename] = tradelink
      end
    end
  end -- of function
end -- of do
    
function addon:TRADE_SKILL_CLOSE()
end

do
  local last_update = 0
  local updater = CreateFrame("Frame", nil, UIParent)
  
  updater:Hide()
  updater:SetScript("OnUpdate",
                    function(self, elapsed)
                      last_update = last_update + elapsed
                      
                      if last_update >= 0.5 then -- make this dependent on profile information
                        local profession = GetTradeSkillLine()
                        
                        if profession ~= "UNKNOWN" then
                        end
                      end
                    end)

  function addon:TRADE_SKILL_UPDATE(...)
    if not updater:IsVisible() then
      last_update = 0
      updater:Show()
    end
  end
end -- of do
